// In diesem Beispiel werden Daten zwischen Flash und Arduino ausgetauscht.
// Arduino schreibt eine XML-Datei, die von Flash gelesen werden kann.
// Flash schickt HTTP-Requests um Daten an den Arduino zu senden.


#include <Servo.h>    // Servo-Library importieren
#include <SPI.h>      // SPI-Library importieren (für Serielle Schnittstelle)
#include <Ethernet.h> // Ethernet-Library importieren (für Ethernet-Schnittstelle)

//Hub (Intern) Netzwerkeinstellungen
//byte mac[] = { 0x90, 0xA2, 0xDA, 0x00, 0x82, 0xCC }; // MAC-Adresse des Ethernet-Shields
//byte ip[] = { 192, 168, 14, 10 };                    // IP-Adresse des Arduinos
//byte gateway[] = { 192, 168, 14, 254 };              // Netzwerk-Gateway für das interne Netzwerk
//byte subnet[]  = { 255, 255, 255, 0 };               // Subnetz-Maske für das interne Netzwerk

//ZUHAUS Netzwerkeinstellungen
byte mac[] = { 0x90, 0xA2, 0xDA, 0x00, 0x82, 0xCC }; // MAC-Adresse des Ethernet-Shields
byte ip[] = { 192, 168, 178, 222 };                  // IP-Adresse des Arduinos
byte gateway[] = { 192, 168, 178, 1 };               // Netzwerk-Gateway für das interne Netzwerk
byte subnet[]  = { 255, 255, 255, 0 };               // Subnetz-Maske für das interne Netzwerk

int sensorPin = 0;   // Poti-Pin ("Temperaturmessung")
int servoPin = 9;    // Servo-Pin ("Heizungssteuerung")
int serverPort = 80; // Port zur Datenübertragung (HTTP -> Port 80)
int msgLength = 100; // maximale Zahl an Zeichen, die eine Anfrage vom Client haben darf

Server server(serverPort);  // Server-Objekt, um Arduino als Webserver zu verwenden
Servo servo;                // Servo-Objekt zum Steuern des Servos

int tmpValue = 0; // Variable zum Speichern der aktuellen "Temperatur"
String readString = String(msgLength); // Variable zum Speichern von Client-Anfragen (z.B. von Flash)

void setup()
{
  Ethernet.begin(mac, ip, gateway, subnet);  // Ethernet-Verbindung starten
  server.begin();  // Server-Funktionen starten
  
  servo.attach(servoPin);  // Servo am Servo-Pin aktivieren

  Serial.begin(9600); // Serielle Übertragung starten
}

void loop()
{
  Client client = server.available();  // potentiell verbundenen Client speichern
  
  if (client)  // wenn ein Client verbunden ist
  {
    while (client.connected()) // solang der Client verbunden ist
    {
      if (client.available())  // wenn der Client eine Nachricht gesendet hat
      {
        char c = client.read(); // lies das nächste byte der Nachricht ein
  
        if (readString.length() < msgLength) // solange die Nachricht kürzer als maximale Zeichenzahl ist
        {
          readString = readString + c; // hänge das eingelesene Zeichen an readString an
        }

        if (c == '\n') // wenn Zeilenumbruch erreicht ist (Ende des HTTP-Requests)
        {  
          if(readString.indexOf("Servo") > -1) // wenn die eingelesene Nachricht das Wort Servo enthält
          {
            // die Nachricht so beschneiden, dass nur noch der übertragene Zahlenwert übrig bleibt und in valueStr speichern
            String valueStr = readString.substring(readString.indexOf("Servo")+5, readString.lastIndexOf(" "));

            char charArray [4];                 // ein neues Character-Array anlegen
            valueStr.toCharArray(charArray, 4); // den eingelesenen Zahlenwert in das Array schreiben
            int value = atoi(charArray);        // das Array in einen Integer-Wert umwandeln und in value speichern

            servo.write(value);  // den Servo auf den übertragenen Zahlenwert drehen
          }
          
          tmpValue = analogRead(sensorPin);  // den Temperatur-Wert vom Sensor-Pin auslesen
          
          client.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");  // XML-Header
          client.println("<body>");          // Start-Tag für XML-"Rumpf"
          client.println("<values>");        // Start-Tag für Werte
          client.print("<temperature>");     // Start-Tag für Temperatur-Wert
          client.print(tmpValue);            // Temperatur-Wert
          client.println("</temperature>");  // End-Tag für Temperatur-Wert
          client.println("</values>");       // End-Tag für Werte
          client.println("</body>");         // End-Tag für XML-"Rumpf"

          readString=""; // readString-Variable entleeren (für neue Nachrichten vorbereiten)

          client.stop(); // die Client-Verbindung beenden
        }
      }
    }
  }
}
